@@ -5,27 +5,31 @@ use warnings;
 
 use Class::Accessor::Inherited::XS::Compat qw/mk_type_accessors mk_inherited_accessors mk_class_accessors mk_varclass_accessors mk_object_accessors/;
 
-use Carp ();
+our $PREFIX = '__cag_';
 
-our $VERSION = '0.34';
-our $PREFIX  = '__cag_';
+BEGIN {
+    our $VERSION = '0.35';
 
-require XSLoader;
-XSLoader::load('Class::Accessor::Inherited::XS', $VERSION);
+    require XSLoader;
+    XSLoader::load('Class::Accessor::Inherited::XS', $VERSION);
+}
+
+use Carp qw/confess/;
+use Class::Accessor::Inherited::XS::Constants;
 
 my $REGISTERED_TYPES = {};
 register_types(
-    inherited       => {installer => sub { _mk_inherited_accessor(@_, 0) }, clone_arg => 1},
-    inherited_ro    => {installer => sub { _mk_inherited_accessor(@_, 1) }, clone_arg => 1},
-    class           => {installer => sub { _mk_class_accessor(@_, 0, 0) },  clone_arg => undef},
-    class_ro        => {installer => sub { _mk_class_accessor(@_, 0, 1) },  clone_arg => undef},
-    varclass        => {installer => sub { _mk_class_accessor(@_, 1, 0) },  clone_arg => undef},
-    varclass_ro     => {installer => sub { _mk_class_accessor(@_, 1, 1) },  clone_arg => undef},
-    object          => {installer => sub { _mk_object_accessor(@_, 0) },    clone_arg => 1},
-    accessors       => {installer => sub { _mk_object_accessor(@_, 0) },    clone_arg => 1}, # alias for object
-    object_ro       => {installer => sub { _mk_object_accessor(@_, 1) },    clone_arg => 1},
-    getters         => {installer => sub { _mk_object_accessor(@_, 1) },    clone_arg => 1}, # alias for object_ro
-    constructor     => {installer => \&_mk_constructor,                     clone_arg => undef},
+    inherited       => {installer => _curry(\&_mk_inherited_accessor, None),       clone_arg => 1},
+    inherited_ro    => {installer => _curry(\&_mk_inherited_accessor, IsReadonly), clone_arg => 1},
+    class           => {installer => _curry(\&_mk_class_accessor, 0, None),        clone_arg => undef},
+    class_ro        => {installer => _curry(\&_mk_class_accessor, 0, IsReadonly),  clone_arg => undef},
+    varclass        => {installer => _curry(\&_mk_class_accessor, 1, None),        clone_arg => undef},
+    varclass_ro     => {installer => _curry(\&_mk_class_accessor, 1, IsReadonly),  clone_arg => undef},
+    object          => {installer => _curry(\&_mk_object_accessor, None),          clone_arg => 1},
+    accessors       => {installer => _curry(\&_mk_object_accessor, None),          clone_arg => 1}, # alias for object
+    object_ro       => {installer => _curry(\&_mk_object_accessor, IsReadonly),    clone_arg => 1},
+    getters         => {installer => _curry(\&_mk_object_accessor, IsReadonly),    clone_arg => 1}, # alias for object_ro
+    constructor     => {installer => \&_mk_constructor,                            clone_arg => undef},
 );
 
 sub import {
@@ -49,7 +53,7 @@ sub import {
             $installer->($class, $accessors, $clone_arg && $accessors);
 
         } else {
-            Carp::confess("Can't understand format for '$type' accessors initializer");
+            confess("Can't understand format for '$type' accessors initializer");
         }
     }
 }
@@ -64,14 +68,16 @@ sub register_type {
     my ($type, $args) = @_;
 
     if (exists $REGISTERED_TYPES->{$type}) {
-        Carp::confess("Type '$type' has already been registered");
+        confess("Type '$type' has already been registered");
     }
 
     if (!exists $args->{installer}) {
         $args->{installer} = sub {
             my ($class, $name, $field) = @_;
             install_inherited_cb_accessor(
-                "${class}::${name}", $field, $PREFIX.$field, $args->{read_cb} // $args->{on_read}, $args->{write_cb} // $args->{on_write}, 0
+                "${class}::${name}", $field, $PREFIX.$field,
+                $args->{read_cb} // $args->{on_read}, $args->{write_cb} // $args->{on_write},
+                $args->{opts} // 0,
             );
         };
     }
@@ -84,10 +90,18 @@ sub register_type {
 #   Functions below are NOT part of the public API
 #
 
+sub _curry {
+    my ($sub, @args) = @_;
+
+    return sub {
+        $sub->(@_, @args);
+    };
+}
+
 sub _type_installer {
     my (undef, $type) = @_;
 
-    my $type_info = $REGISTERED_TYPES->{$type} or Carp::confess("Don't know how to install '$type' accessors");
+    my $type_info = $REGISTERED_TYPES->{$type} or confess("Don't know how to install '$type' accessors");
     return ($type_info->{installer}, $type_info->{clone_arg});
 }
 
@@ -98,9 +112,9 @@ sub _mk_inherited_accessor {
 }
 
 sub _mk_class_accessor {
-    my ($class, $name, $default, $is_varclass, $is_readonly, $is_weak) = @_;
+    my ($class, $name, $default, $is_varclass, $flags) = @_;
 
-    install_class_accessor("${class}::${name}", $default, $is_varclass, ($is_readonly ? 1 : 0) | ($is_weak ? 2 : 0));
+    install_class_accessor("${class}::${name}", $default, $is_varclass, $flags);
 }
 
 sub _mk_object_accessor {
@@ -289,7 +303,7 @@ and $__PACKAGE__::accessor variable. While the stored value is still accessible
 
 Copyright (C) 2009 by Vladimir Timofeev
 
-Copyright (C) 2014-2016 by Sergey Aleynikov
+Copyright (C) 2014-2018 by Sergey Aleynikov
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.10.1 or,
