@@ -1,7 +1,7 @@
 package ExtraInheritedGroups;
 use strict;
 use warnings;
-use base 'AccessorInstaller';
+use parent 'AccessorInstaller';
 
 __PACKAGE__->mk_inherited_accessors('basefield');
 __PACKAGE__->basefield('your extra base!');
